'***********************************************************************
'
'    Filename        :     Vorwahlzhler		
'    File Version    :     1.0
'    Datum           :     02.03.2008             	
'    Autor           :     Peter Verderber		
'    Funktion        :     Vorwahlzhler mit Vor- und Rckwrtserkennung		
'    Letzte Revision :     1.0
'    Datum           :     02.03.2008           
'    Ziel PIC        :     16F628A	
'    Compiler        :     iL_BAS16 - Professional V5.7-9  18.07.2007 
'    
'***********************************************************************
'*****                Definitionen und Konstanten                  *****
'***********************************************************************

Date
Time

$LIST INHX8    'erzeugt eine OBJ-Datei im Intel-Hex8-Format (default)
'$LIST OBJ2HEX  'das Hexfile bekommt anstatt OBJ die Dateiendung HEX

$NCALDEF
         
define device 16f628a,wdt_off,irc_osc,cmcfg7,MCLR_INT,OSC2_IO,T0CS_INT     
                  
tris ra,%00000111
tris rb,%11000000
                   
'$DEBUG 3,ra,4                    'Debug Pin     

define pulse_soll  = $30 as word  'Anzahl SOLL_Impulse
define pulse_sollL = $30 as byte  'Zugriff Lowbyte  der 16-Bit-Var
define pulse_sollH = $31 as byte  'Zugriff Highbyte der 16-Bit-Var

define pulse_ist   = auto as word 'Anzahl IST_Impulse

define stelle      = auto as byte 'Wertigkeit der Stelle 10^0 - 10^4 
define var_z       = auto as byte 'Zhlvariable
define var_z0      = auto as byte 'Zhlvariable Einer
define var_z1      = auto as byte 'Zhlvariable Zehner
define var_z2      = auto as word 'Zhlvariable Hunderter 
define var_z3      = auto as word 'Zhlvariable Tausender
define var_z4      = auto as word 'Zhlvariable Zehntausender

define schranke1   = rb,7         'PORT RB7 fr Sensor1
define schranke2   = rb,6         'PORT RB6 fr Sensor2

define up_taste    = ra,0         'Up Taste
define enter_taste = ra,1         'Enter Taste
define mode_taste  = ra,2         'Mode Taste          

define relais      = ra,3         'Relais ein / aus  

goto main                         'Start Hauptprogramm

'***********************************************************************
'*****                   Unterprogramme                            *****
'***********************************************************************

init:
  wait 1000                            'LCD-Initialisierung 
  lcdinit rb,2,16,
  lcdwrite 1,1," Vorwahlzaehler "      'Begrungstext
  lcdwrite 2,1,"PVC - electronic"
  'lcdwrite 2,1,"-DC2MR-DARC-CQDL"
  wait 2500
    
  let pulse_ist  = 0                   'Ist-Wert  = 0
  let pulse_soll = 0                   'Soll-Wert = 0 
  res relais                           'Realis abgefallen
  lcdclear
  
  return

'-----------------------------------------------------------------------
'                            Anzeigetexte                            
'-----------------------------------------------------------------------

lcd_text11:
  lcdwrite 1,1,"STBY"
  return 

lcd_text12:
  lcdwrite 1,1,"ON  "
  return 

lcd_text13:
  lcdwrite 1,7,"SOLL"
  return 

lcd_text14:
  lcdwrite 1,14,"IST"
  return 

lcd_text15:
  lcdwrite 1,1,"Sollwert"
  return

lcd_text20:
  lcdwrite 2,1,"ON "
  return
  
lcd_text21:
  lcdwrite 2,1,"OFF"
  return 

lcd_text22:
  lcdwrite 2,1,"Stelle"
  return 

'-----------------------------------------------------------------------
'                  Vorwahlzhler Sollwert einstellen                            
'-----------------------------------------------------------------------

set_counter:                 'Soll-Wert setzen
  let stelle  = 0
  let var_z   = 0
  let var_z0  = 0
  let var_z1  = 0
  let var_z2  = 0
  let var_z3  = 0
  let var_z4  = 0
    
  lcdclear      
  gosub lcd_text15           '"Sollwert" 
  gosub lcd_text22           '"Stelle"
                                       
  goto set_sollwert       
  
  lcdclear
  gosub lcd_text13           '"SOLL"      
  gosub lcd_text14           '"IST"
  gosub lcd_text21           '"OFF"
                  
  return        
      
  set_sollwert:              'Stellen setzen
        
    on stelle gosub ZT_Stelle,T_Stelle,H_Stelle,Z_Stelle,E_Stelle
    
    wait 100
    if enter_taste = 0 then inc stelle
       
    let pulse_soll = var_z0 + var_z1 + var_z2 + var_z3 + var_z4 
    lcdwrite 1,12,pulse_soll
    gosub store_pulse_soll
    
    if stelle > 4 then return    
    
    goto set_sollwert
    
    ZT_Stelle:                         'Stelle 10^4 setzen
      lcdwrite 2,8,"ZT"                'mit Bereichsberprfung
      gosub inc_soll
      let var_z4 = var_z * 10000
      if var_z > 5 then goto ZT_Stelle
      return   
    
    T_Stelle:                          'Stelle 10^3 setzen
      lcdwrite 2,8,"T " 
      gosub inc_soll
      let var_z3 = var_z * 1000
      return  
    
    H_Stelle:                          'Stelle 10^2 setzen
      lcdwrite 2,8,"H " 
      gosub inc_soll
      let var_z2 = var_z * 100
      return 
    
    Z_Stelle:                          'Stelle 10^1 setzen
      lcdwrite 2,8,"Z " 
      gosub inc_soll
      let var_z1 = var_z * 10
      return
    
    E_Stelle:                          'Stelle 10^0 setzen
      lcdwrite 2,8,"E "
      gosub inc_soll 
      let var_z0 = var_z  
      return
    
    inc_soll:                          'var_z addieren 
      lcdwrite 2,12,var_z              'mit Bereichsberprfung 
      wait 350
      if up_taste = 0 then inc var_z     
      if  var_z > 9 then let var_z = 0
      if enter_taste = 1 then goto inc_soll
      return  
       
'-----------------------------------------------------------------------
'        Sollwert im EEPROM speichern bzw. aus EEPROM auslesen                           
'-----------------------------------------------------------------------     

store_pulse_soll:                      'abspeichern der 16-Bit-Variable,
  write 5,pulse_sollL                  'zuerst das Low-Byte 
  wait 15
  write 6,pulse_sollH                  'dann das High-Byte 
  wait 15
  return       

read_pulse_soll:                       'lesen der 16-Bit-Variable,
  read 5,pulse_sollL                   'zuerst das Low-Byte 
  wait 15
  read 6,pulse_sollH                   'dann das High-Byte 
  wait 15
  return       

'-----------------------------------------------------------------------
'                       Relais - Ansteuerung                           
'----------------------------------------------------------------------- 

Vorwahl_erreicht:            'Relais angezogen 
  set relais 
  gosub lcd_text12           '"ON  "
  return


'***********************************************************************
'*****                     Hauptprogramm                           *****
'***********************************************************************

main:
  gosub init                           'Initialisierung aufrufen
  
  gosub read_pulse_soll                'Vorwahl aus dem EEPROM lesen
  if pulse_soll =      0 then gosub set_counter 
  if pulse_soll >  59999 then gosub set_counter
 

 
count_standby:                         'Start Vorwahlzhler Standby-Mode          
  lcdwrite 1,1,"STBY  SOLL   IST"      
  lcdwrite 2,1,"OFF  "
  lcdwrite 2, 6,pulse_soll
  lcdwrite 2,12,pulse_ist
  
  if pulse_ist >= pulse_soll then set relais else res relais 
  if pulse_ist >= pulse_soll then gosub lcd_text20 else gosub lcd_text21   
  
  if enter_taste = 0 then let pulse_ist = 0      'Vorwahlzhler reset
  if mode_taste  = 0 then gosub set_counter      'Vorwahlzhler set
  if up_taste    = 0 then goto count_loop        'Start Zhler  
   
goto count_standby
 
       

count_loop:                            'Start Vorwahlzhler Zhl-Mode 
   lcdwrite 1,1,"ON  "    
    
  Sensor_Abfrage:                      'Abfrage Sensoren und Enter-Taste 
    if schranke1   = 1 then goto inc_pulse       
    if schranke2   = 1 then goto dec_pulse
    if enter_taste = 0 then goto count_standby
    goto Sensor_Abfrage
         
  inc_pulse:                                     'Impulse aufaddieren
    inc pulse_ist                                'mit Bereichsberprfung     
    if pulse_ist > 59999 then let pulse_ist = 0
    goto anzeige_IST
        
  dec_pulse:                                     'Impulse subtrahieren     
    dec pulse_ist                                'mit Bereichsberprfung      
    if pulse_ist =     0 then let pulse_ist = 0
    if pulse_ist > 59999 then let pulse_ist = 0
    goto anzeige_IST
  
  anzeige_IST:
    lcdwrite 2, 6,pulse_soll
    lcdwrite 2,12,pulse_ist
    
    if pulse_ist >= pulse_soll then set relais else res relais 
    if pulse_ist >= pulse_soll then gosub lcd_text20 else gosub lcd_text21  
    
goto count_loop

end                                              ' Ende gut alles gut